# Will be used for generic humanoid characters that all have the same skeleton+animations
# Basically let's you split upper and lower body, have them aim separate directions and animate separately

extends AnimationTree

onready var last_move_dir = get_parent().global_transform.basis.z
onready var skeleton = get_parent().get_node("Scene Root/Armature/Skeleton")
onready var playback = get("parameters/playback")

func update_dead():
	if playback.get_current_node() != "dead":
		playback.start("dead")
		pass

func update_body(move_dir, aim_dir, eating=false):
	if eating and playback.get_current_node() != "eat_state":
		playback.start("eat_state")
	elif not eating:
		if playback.get_current_node() != "main_state":
			playback.start("main_state")
		update_legs(move_dir, aim_dir)
		face_upper_body_in_dir(move_dir, aim_dir)

func update_legs(move_dir, aim_dir):
	var mov_spd = move_dir.length()
	if mov_spd == 0:
		update_legs_standing(aim_dir)
		update_move_speed(1.0) #want idle anims to play at regular speed
	else:
		update_legs_running(move_dir, aim_dir)
		last_move_dir = move_dir
		update_move_speed(mov_spd)

func update_move_speed(mv_spd):
	set("parameters/main_state/move_time_scale/scale", mv_spd / 8.0)

func update_legs_standing(aim_dir):
	set("parameters/main_state/movement_blend_space/blend_position", 0.0)
	if aim_dir.dot(last_move_dir) < 0.2:
		set("parameters/main_state/hop_one_shot/active", true)
		face_lower_body_in_dir(aim_dir)
		last_move_dir = aim_dir
	else:
		face_lower_body_in_dir(last_move_dir)

func update_legs_running(move_dir, aim_dir):
	if aim_dir.dot(move_dir) < 0.0:
		face_lower_body_in_dir(-move_dir) #run backwards
		set("parameters/main_state/movement_blend_space/blend_position", -1.0)
	else:
		face_lower_body_in_dir(move_dir) #run forwards
		set("parameters/main_state/movement_blend_space/blend_position", 1.0)

func face_lower_body_in_dir(dir):
	var bone = skeleton.find_bone("lowerbody")
	var bone_transform = skeleton.get_bone_rest(bone)
	var rotate_amnt = vector3_to_angle(dir) - get_parent().global_transform.basis.get_euler().y
	bone_transform = bone_transform.rotated(Vector3(0,1,0), rotate_amnt)
	skeleton.set_bone_global_pose(bone, bone_transform)

func face_upper_body_in_dir(move_dir, aim_dir): #sets upper_body to 0
	
	var bone = skeleton.find_bone("upperbody")
	var bone_transform = skeleton.get_bone_global_pose(bone)
	var r = 0
	if move_dir == Vector3():
		r = atan2(-last_move_dir.x, last_move_dir.z)
	else:
		if move_dir.dot(aim_dir) > 0:
			r = atan2(-move_dir.x, move_dir.z)
		else:
			r = atan2(move_dir.x, -move_dir.z)
	var rotate_amnt = r + get_parent().global_transform.basis.get_euler().y
	bone_transform = bone_transform.rotated(Vector3(0,1,0), rotate_amnt)
	skeleton.set_bone_global_pose(bone, bone_transform)

func reload():
	set("parameters/main_state/upper_body_control/0/rifle_control/active", true)
	

func vector3_to_angle(vec):
	return atan2(vec.x, vec.z)