# controls groups of bandits and coordinates their movements
# video in progress explaining how this works, will link when done

extends Spatial

var group_info = [] # {obj: <obj>, start_pos : Vector3()}
const FLANK_DISTANCE = 15
const MAX_TIME_ENEMY_OUT_OF_RANGE = 3.0
const MAX_DIST_ENEMY_MOVES_FROM_GOAL_BEFORE_RESET = 2.0

var cur_time_enemy_out_of_range = 0.0
var currently_flanking = false
var goal_pos = Vector3()
var enemy = null
export var disable = false

func _ready():
	if disable:
		return
	for child in get_children():
		if has_signal(child, "spotted_enemy"):
			child.connect("spotted_enemy", self, "init_flank")
			child.connect("reached_point", self, "unit_reached_point")
			child.override = true
			group_info.append({"obj": child, "quadrant_path": [], "farthest_point_reached": -1})

func has_signal(obj, sig_name):
	for sig in obj.get_signal_list():
		if sig.name == sig_name:
			return true
	return false

func _process(delta):
	if enemy == null:
		return
	var enemy_in_sight = true
	for unit_info in group_info:
		if !unit_info.obj.enemy_in_sight_area:
			enemy_in_sight = false
		else:
			cur_time_enemy_out_of_range = 0.0
	if !enemy_in_sight:
		cur_time_enemy_out_of_range += delta
	if currently_flanking and cur_time_enemy_out_of_range > MAX_TIME_ENEMY_OUT_OF_RANGE:
		for unit_info in group_info:
			unit_info.obj.switch_to_rest_state()
			unit_info.farthest_point_reached = -1
			currently_flanking = false
	var e_pos = enemy.global_transform.origin
	if currently_flanking and e_pos.distance_squared_to(goal_pos) > pow(MAX_DIST_ENEMY_MOVES_FROM_GOAL_BEFORE_RESET, 2):
		generate_paths_for_units()

func init_flank(target):
	if currently_flanking:
		return
	#print("starting flank")
	cur_time_enemy_out_of_range = 0.0
	currently_flanking = true
	enemy = target
	for unit_info in group_info:
		unit_info.obj.switch_to_chase_state()
	
	var e_pos = enemy.global_transform.origin
	goal_pos = e_pos
	var points = get_quad_points(e_pos)
	var quadrants = [[], [], [], []] 
	var group_info_ind = 0
	# separate units into four groups relative to player: NE, SE, SW, NW
	for unit_info in group_info:
		var unit = unit_info.obj
		var b_pos = unit.global_transform.origin
		var min_dis = b_pos.distance_squared_to(points[0])
		var min_ind = 0
		for i in range(3):
			var dis = b_pos.distance_squared_to(points[i+1])
			if dis < min_dis:
				min_dis = dis
				min_ind = i
		quadrants[min_ind].append(group_info_ind) # index of unit in group_info
		group_info_ind += 1
	
	# decide what quadrants each unit should go through to reach player
	for i in range(4):
		var opp_ind = (i + 2) % 4
		var units_in_this_quad = quadrants[i].size()
		var units_in_opp_quad = quadrants[opp_ind].size()
		var avg = int((units_in_this_quad + units_in_opp_quad) / 2)
		if avg < units_in_this_quad: # decide if need to send units to opposite quadrant for flank
			for j in range(units_in_this_quad - avg): # send them over
				# calculate path they should take, which adjacent quadrant should they go through?
				var unit_to_send_info = group_info[quadrants[i][j]]
				var left_quad_ind = (i + 1) % 4
				var right_quad_ind = (i - 1) % 4
				var unit_to_send = unit_to_send_info.obj
				var unit_to_send_pos = unit_to_send.global_transform.origin
				var dis_to_left = points[left_quad_ind].distance_squared_to(unit_to_send_pos)
				var dis_to_right = points[right_quad_ind].distance_squared_to(unit_to_send_pos)
				
				var first_stop = left_quad_ind
				if dis_to_left > dis_to_right:
					first_stop = right_quad_ind
				group_info[quadrants[i][j]].quadrant_path = [first_stop, opp_ind]
				#print("path: ", first_stop, " ", opp_ind)
	
	generate_paths_for_units()

func generate_paths_for_units():
	#print("updated paths")
	var e_pos = enemy.global_transform.origin
	for unit_info in group_info:
		var unit = unit_info.obj
		var unit_pos = unit.global_transform.origin
		var points_list = [unit_pos]
		
		var quad_points = []
		var q_was_set = false
		for quadrant_ind in unit_info.quadrant_path:
			if quadrant_ind > unit_info.farthest_point_reached:
				var q_point = get_quad_point(e_pos, quadrant_ind)
				quad_points.append(q_point)
				if !q_was_set:
					#print("set on ", quadrant_ind)
					q_was_set = true
					unit_info.obj.alert_when_reached_point = q_point
		
		if quad_points.size() > 1:
			#check if unit has made it past the first quad point, if so skip it
			var d = (quad_points[0] - unit_pos).dot(quad_points[1] - unit_pos)
			if d < 0:
				points_list.append(quad_points[1])
			else:
				points_list += quad_points
		
		points_list.append(e_pos)
		var new_path = generate_path_through_points(points_list)
		unit.path = new_path
		#print(new_path)
	goal_pos = e_pos

func generate_path_through_points(points_list):
	if points_list.size() < 2:
		return []
	var path = PoolVector3Array()
	var last_point = points_list[0]
	
	for i in range(points_list.size() - 1):
		path.append_array(get_parent().get_simple_path(last_point, points_list[i+1]))
		last_point = points_list[i+1]
	
	return path

var n_points = [
	Vector3(1, 0, 1), #NE
	Vector3(-1, 0, 1), #SE
	Vector3(-1, 0, -1), #SW
	Vector3(1, 0, -1) #NW
]

func get_quad_point(e_pos, quad_ind):
	return n_points[quad_ind] * FLANK_DISTANCE + e_pos
	
func get_quad_points(e_pos):
	var points = [
		e_pos + n_points[0],
		e_pos + n_points[1],
		e_pos + n_points[2],
		e_pos + n_points[3]
	]
	return points

func get_simple_path(start_pos, end_pos, optimize=true):
	return get_parent().get_simple_path(start_pos, end_pos, optimize)

func unit_reached_point(obj):
	for unit_info in group_info:
		if unit_info.obj == obj:
			unit_info.farthest_point_reached += 1
			#print("reached ",unit_info.farthest_point_reached)
			if unit_info.farthest_point_reached + 1 < unit_info.quadrant_path.size():
				obj.alert_when_reached_point = get_quad_point(enemy.global_transform.origin, unit_info.farthest_point_reached + 1)
			else:
				obj.alert_when_reached_point = null
			break