# used by player to control weapons, hotkeys, and eating

extends Spatial

onready var hand = get_parent().get_node("Graphics/Scene Root/Armature/Skeleton/Hand")
onready var health_manager = $HealthManager
onready var anim = get_parent().get_node("Graphics/AnimationTree")
var weapon_slots = {
	0: "",
	1: "",
	2: "",
	3: "",
}
onready var cur_weapon = null
var cur_weapon_slot = 0
var time_of_interrupt = 0
var time_cant_eat_for_after_interrupt = 0.5
var fire_point = Vector3()

var dont_update_cursor = false

func _ready():
	load_cur_weapon()

func switch_weapon(slot):
	if slot < 0 or slot >= weapon_slots.size():
		return
	cur_weapon_slot = slot
	load_cur_weapon()
	
func load_cur_weapon():
	# TODO get weapon type and set animations accordingly
	if cur_weapon != null:
		cur_weapon.queue_free()
		cur_weapon = null
	var weapon_id = weapon_slots[cur_weapon_slot]
	if weapon_id == "":
		weapon_id = "knife"
	var new_wep = ItemDB.create_weapon(weapon_id)
	hand.add_child(new_wep)
	cur_weapon = new_wep
	cur_weapon.global_transform = hand.global_transform
	cur_weapon.set_objects_to_ignore([get_parent()])
	cur_weapon.held_by = "Player"
	cur_weapon.fire_point = fire_point
	update_crosshair()
	cur_weapon.connect("reloading", anim, "reload")

func update_crosshair():
	if dont_update_cursor:
		return
	CursorManager.set_cursor(cur_weapon.crosshair)
	
func equip_weapon(slot, weapon_id):
	if slot < 0 or slot >= weapon_slots.size():
		return
	weapon_slots[slot] = weapon_id
	if cur_weapon_slot == slot:
		load_cur_weapon()

func single_attack():
	cur_weapon.single_attack()
	interrupt()

func auto_attack():
	cur_weapon.auto_attack()
	interrupt()

func reload():
	cur_weapon.reload()
	interrupt()

func heal(amnt):
	health_manager.heal(amnt)

func damage(amnt, dir):
	health_manager.damage(amnt, dir)

func eat():
	if is_interrupted():
		return
	cur_weapon.hide()
	health_manager.eat()

func stop_eating():
	cur_weapon.show()
	health_manager.stop_eating()

func is_eating():
	return health_manager.eating

func is_interrupted():
	return time_of_interrupt - time_cant_eat_for_after_interrupt > OS.get_ticks_msec() / 1000.0

func at_full_health():
	return health_manager.cur_health == health_manager.max_health

func interrupt():
	time_of_interrupt = OS.get_ticks_msec() / 1000.0
	stop_eating()

func stop_all():
	stop_eating()
	cur_weapon.stop_reloading()