extends Spatial

var footsteps = [
	"res://characters/player/audio/PM_SDNG_Single_Step_Footstep_19.wav",
	"res://characters/player/audio/PM_SDNG_Single_Step_Footstep_46.wav"
]

var puddle_splash = preload("res://environment/puddles/PuddleSplash.tscn")

onready var audio_player = $AudioPlayer
onready var splash_particles = $SplashParticles
onready var ground_detector = $GroundDetector

const steps_sfx_path = "res://characters/player/audio/"
var dirt_steps = [
	"step_dirt1.wav", 
	"step_dirt2.wav",
	"step_dirt4.wav", 
	"step_dirt5.wav",
	"step_dirt6.wav"
]
var puddle_steps = [
	"step_puddle1.wav",
	"step_puddle2.wav",
	"step_puddle3.wav"
]
var last_pos = Vector3()

enum Ground {
	REGULAR,
	WATER
}
var ground_under_player = Ground.REGULAR

func ready():
	last_pos = global_transform.origin
	
	var new_footsteps = []
	for footstep in footsteps:
		new_footsteps.append(load(footstep))
	footsteps = new_footsteps

func _physics_process(_delta):
	
	if ground_detector.is_colliding():
		if ground_detector.get_collider().name == "Water":
			ground_under_player = Ground.WATER
		else:
			ground_under_player = Ground.REGULAR
	else:
		ground_under_player = Ground.REGULAR
	splash_particles.emitting = last_pos != global_transform.origin and ground_under_player == Ground.WATER
	last_pos = global_transform.origin
	

func play_step(spawn_splash: bool):
	#print(OS.get_ticks_msec() / 1000.0)
	if ground_under_player == Ground.WATER:
		audio_player.stream = load(steps_sfx_path + puddle_steps[randi() % puddle_steps.size()])
	else:
		audio_player.stream = load(steps_sfx_path + dirt_steps[randi() % dirt_steps.size()])
	#stop()
	audio_player.play()
	if spawn_splash and ground_under_player == Ground.WATER:
		var puddle_splash_instance = puddle_splash.instance()
		get_tree().get_root().add_child(puddle_splash_instance)
		puddle_splash_instance.global_transform.origin = global_transform.origin
	