extends Spatial

# generic health manager, handles amount of health, health regen, 
# and eating nearby corpses to heal
# TODO add performance stuff to disable body checks for eating if char is not an eater
onready var meat_detector = $MeatDetector  # area masked to meat layer

onready var blood_splatter = preload("res://effects/BloodSplatter/BloodSplatter.tscn")
#onready var ground_blood_splatter = preload("res://effects/BloodSplatter/GroundBloodSplatter.tscn")
#onready var blood_decal_spray = preload("res://effects/BloodSplatter/ProjectedBloodDecal.tscn")

export var max_health = 100
export var health_regen_rate = 2 # amount healed per second
export var eat_heal_amnt = 1 # amount healed per bite
export var eat_rate = 2 # bites per second

var cur_health = 100
var cur_regen = 0.0 # used to keep health increase as int
var cur_eat = 0.0 # time since started eating
var eating = false
var dead = false

signal damaged # sends (current health, max_health)
signal health_changed # sends (current health, max_health)
signal died

func _process(delta):
	if dead:
		return
	cur_regen += health_regen_rate * delta
	while cur_regen > 1:
		cur_regen -= 1
		heal(1)
	
	if eating:
		cur_eat += eat_rate * delta
		if cur_eat > 1:
			cur_eat -= 1
			heal(eat_heal_amnt)
	else:
		cur_eat = 0.0

func update_stats(new_stats: Dictionary):
	max_health = new_stats.hp_max
	health_regen_rate = new_stats.hp_regen
	eat_heal_amnt = new_stats.digestion
	if cur_health > max_health:
		cur_health = max_health
	emit_signal("health_changed", cur_health, max_health)

func heal(amnt: int):
	cur_health += amnt
	if cur_health > max_health:
		cur_health = max_health
	emit_signal("health_changed", cur_health, max_health)

func damage(amnt: int, dir: Vector3):
	cur_health -= amnt
	
	#create graphics
	var bs = blood_splatter.instance()
	get_tree().get_root().add_child(bs)
	bs.global_transform.origin = global_transform.origin + Vector3(0, 1, 0)
	bs.global_rotate(Vector3(0, 1, 0), atan2(dir.x, dir.z))
	bs.init()
	if cur_health < 0:
		cur_health = 0
		dead = true
		$MeatDetector.set_collision_layer_bit(6, true) # 6 is meat index
		emit_signal("died")
	
	emit_signal("damaged", cur_health, max_health)
	emit_signal("health_changed", cur_health, max_health)

func eat():
	if dead or not meat_nearby():
		return
	if cur_health < max_health:
		eating = true

func meat_nearby():
	return meat_detector.get_overlapping_areas().size() > 0

func get_nearest_meat():
	if meat_nearby():
		return meat_detector.get_overlapping_areas()[0]
	else:
		return null

func stop_eating():
	eating = false
	cur_eat = 0.0