extends InventoryContainer

# non grid based container

var items = []

func insert_item(item: ItemUI) -> bool:
	var item_pos = item.rect_global_position + item.rect_size / 2
	if not get_global_rect().has_point(item_pos): 
		return false
	var item_top_left_local_pos = item.rect_global_position - rect_global_position 
	var item_bot_right_local_pos = item_top_left_local_pos + item.rect_size 
	var offset = Vector2()
	if item_top_left_local_pos.x < 0:
		offset.x = -item_top_left_local_pos.x
	if item_top_left_local_pos.y < 0:
		offset.y = -item_top_left_local_pos.y
	if item_bot_right_local_pos.x > rect_size.x:
		offset.x = rect_size.x - item_bot_right_local_pos.x
	if item_bot_right_local_pos.y > rect_size.y:
		offset.y = rect_size.y - item_bot_right_local_pos.y
	item.rect_global_position += offset
	items.append(item)
	return true

func grab_item(pos: Vector2) -> ItemUI:
	var item = get_item_under_pos(pos)
	if item == null:
		return null
	items.remove(items.find(item))
	return item

func peek_item(pos: Vector2) -> ItemUI:
	return get_item_under_pos(pos)

func get_item_under_pos(pos: Vector2) -> ItemUI:
	for item in items:
		if item.get_global_rect().has_point(pos):
			return item
	return null

func _save():
	var inv_state = []
	for item in items:
		var item_state = {}
		item_state["id"] = item.id
		item_state["rotated"] = item.rotated
		var offset_normalized = ((item.rect_global_position + item.rect_size / 2.0) - (rect_global_position + rect_size / 2.0)) / (rect_size / 2.0)
		item_state["x"] = offset_normalized.x
		item_state["y"] = offset_normalized.y
		inv_state.append(item_state)
	return inv_state 

func _load(inv_state, item_parent):
	for item in items:
		item.queue_free()
	items = []
	var items_loaded = []
	for item_state in inv_state:
		var item = ItemDB.create_item_for_ui(item_state.id, item_parent)
		if item_state.rotated:
			item.rotate(item.rect_global_position)
		var offset_normalized = Vector2(item_state.x, item_state.y)
		var new_pos = rect_global_position + (offset_normalized + Vector2(1,1)) * rect_size / 2.0  - item.rect_size/ 2.0
		item.rect_global_position = new_pos
		items.append(item)
		items_loaded.append(item)
	return items_loaded