extends InventoryContainer

var items = []

var grid = {}
var cell_size = 64
var grid_width = 3
var grid_height = 5

signal inserted_item
signal grabbed_item
signal hover

func _ready():
	#var s = get_grid_size(self) # todo set this is export, instance ui
	#grid_width = s.x
	#grid_height = s.y
	
	for x in range(grid_width):
		grid[x] = {}
		for y in range(grid_height):
			grid[x][y] = false

func insert_item(item: ItemUI) -> bool:
	var item_pos = item.rect_global_position + Vector2(cell_size / 2, cell_size / 2)
	var g_pos = pos_to_grid_coord(item_pos)
	var item_size = get_grid_size(item)
	if is_grid_space_available(g_pos.x, g_pos.y, item_size.x, item_size.y):
		item.get_parent().remove_child(item)
		$Items.add_child(item)
		set_grid_space(g_pos.x, g_pos.y, item_size.x, item_size.y, true)
		item.rect_global_position = rect_global_position + Vector2(g_pos.x, g_pos.y) * cell_size
		items.append(item)
		emit_signal("inserted_item", item)
		return true
	else:
		return false

func grab_item(pos: Vector2) -> ItemUI:
	var item = get_item_under_pos(pos)
	if item == null:
		return null
	
	var item_pos = item.rect_global_position + Vector2(cell_size / 2, cell_size / 2)
	var g_pos = pos_to_grid_coord(item_pos)
	var item_size = get_grid_size(item)
	set_grid_space(g_pos.x, g_pos.y, item_size.x, item_size.y, false)
	
	items.remove(items.find(item))
	emit_signal("grabbed_item", item)
	return item

func peek_item(pos: Vector2) -> ItemUI:
	var item = get_item_under_pos(pos)
	emit_signal("hover", item)
	return item

func pos_to_grid_coord(pos: Vector2) -> Dictionary:
	var local_pos = pos - rect_global_position
	var results = {}
	results.x = int(local_pos.x / cell_size)
	results.y = int(local_pos.y / cell_size)
	return results

func grid_coord_to_pos(grid_coord: Dictionary):
	return rect_global_position + Vector2(grid_coord.x, grid_coord.y) * cell_size

func get_grid_size(item: ItemUI) -> Dictionary:
	var results = {}
	results.x = item.x_size
	results.y = item.y_size
	return results

func is_grid_space_available(x: int, y: int, w: int, h: int) -> bool:
	if x < 0 or y < 0:
		return false
	if x + w > grid_width or y + h > grid_height:
		return false
	for i in range(x, x + w):
		for j in range(y, y + h):
			if grid[i][j]:
				return false
	return true

func set_grid_space(x: int, y: int, w: int, h: int, state: bool):
	for i in range(x, x + w):
		for j in range(y, y + h):
			grid[i][j] = state

func get_item_under_pos(pos: Vector2) -> ItemUI:
	for item in items:
		if item.get_global_rect().has_point(pos):
			return item
	return null

func pickup_item(item: ItemUI) -> bool:
	if insert_item_at_first_available_spot(item):
		return true
	var tmp_rotation_point = item.rect_global_position
	item.rotate(tmp_rotation_point)
	if insert_item_at_first_available_spot(item):
		return true
	item.rotate(tmp_rotation_point)
	return false

func insert_item_at_first_available_spot(item: ItemUI) -> bool:
	for y in range(grid_height):
		for x in range(grid_width):
			if !grid[x][y]:
				item.rect_global_position = rect_global_position + Vector2(x, y) * cell_size
				if insert_item(item):
					return true
	return false

func get_all_items() -> Array:
	return items

# inv_state is a list with elements in format: {"id": item_id, "x": 0, "y":0, "rotated": false}
func _save() -> Dictionary:
	var inv_state = []
	for item in items:
		var item_state = {}
		item_state["id"] = item.id
		item_state["rotated"] = item.rotated
		var item_pos = item.rect_global_position + Vector2(cell_size / 2, cell_size / 2)
		var grid_pos = pos_to_grid_coord(item_pos)
		item_state["x"] = grid_pos.x
		item_state["y"] = grid_pos.y
		if item.hotkey_slot_id >= 0:
			item_state["hotkey_slot_id"] = item.hotkey_slot_id
		inv_state.append(item_state)
	return inv_state

func _load(inv_state, _item_parent):
	for item in items:
		if not is_instance_valid(item):
			continue
		item.clear_hotkey()
		item.queue_free()
	items = []
	set_grid_space(0, 0, grid_width, grid_height, false)
	var items_loaded = []
	for item_state in inv_state:
		var item = ItemDB.create_item_for_ui(item_state.id, $Items)
		if item_state.rotated:
			item.rotate(item.rect_global_position)
		
		item.rect_global_position = grid_coord_to_pos({"x": item_state.x, "y": item_state.y})
		if not is_grid_space_available(item_state.x, item_state.y, item.x_size, item.y_size):
			print("space conflict in grid")
			item.queue_free()
		else:
			set_grid_space(item_state.x, item_state.y, item.x_size, item.y_size, true)
			items.append(item)
			items_loaded.append(item)
			emit_signal("inserted_item", item)
			if "hotkey_slot_id" in item_state:
				get_parent().get_node("Hotkeys").get_child(int(item_state.hotkey_slot_id)).insert_item(item)
	return items_loaded