extends InventoryContainer

# ground

onready var bag = $Bag
onready var items_in_world_detector = get_parent().get_parent().get_node("ItemDetector") #TODO mask on item layer

var item_detector_range = 0
func _ready():
	item_detector_range = items_in_world_detector.get_node("CollisionShape").shape.extents.x

func insert_item(item: ItemUI) -> bool:
	var inserted_successfully = bag.insert_item(item)
	if inserted_successfully:
		place_item_on_ground(item)
	return inserted_successfully

func grab_item(pos: Vector2) -> ItemUI:
	var item_grabbed = bag.grab_item(pos)
	if item_grabbed != null:
		pick_up_item_from_ground(item_grabbed)
	return item_grabbed

func peek_item(pos: Vector2) -> ItemUI:
	return bag.peek_item(pos)

func load_nearby_items_on_ground():
	for item in bag.items:
		item.queue_free()
	bag.items = []
	var nearby_items = items_in_world_detector.get_overlapping_areas()
	for item in nearby_items:
		var new_item = ItemDB.create_item_for_ui(item.id, get_parent())
		new_item.item_world_ref = item
		#new_item.rect_global_position = bag.rect_global_position # todo place in bag relative to world pos
		new_item.rect_global_position = .9 * item_world_to_bag_pos(item) + new_item.rect_size / 2.0
		if not bag.insert_item(new_item): 
			new_item.rect_global_position = bag.rect_global_position + new_item.rect_size
			bag.items.append(new_item)

func item_world_to_bag_pos(item: Spatial) -> Vector2:
	var item_local_world_pos = item.global_transform.origin - items_in_world_detector.global_transform.origin
	var item_local_world_pos_2d = -Vector2(item_local_world_pos.x, item_local_world_pos.z)
	item_local_world_pos_2d.x = clamp(item_local_world_pos_2d.x, -item_detector_range, item_detector_range)
	item_local_world_pos_2d.y = clamp(item_local_world_pos_2d.y, -item_detector_range, item_detector_range)
	var item_local_pos_scaled = 0.5 * item_local_world_pos_2d / item_detector_range  + Vector2(0.5, 0.5) # gives v2 values of 0 to 1
	var item_bag_pos = item_local_pos_scaled * rect_size + rect_global_position
	return item_bag_pos

func item_bag_to_world_pos(item: ItemUI) -> Vector3:
	var item_local_pos = item.rect_global_position + item.rect_size / 2.0 - rect_global_position
	var item_local_pos_scaled = 2.0 * item_local_pos / rect_size - Vector2(1.0, 1.0) # gives -1 to 1
	var item_world_pos = items_in_world_detector.global_transform.origin - Vector3(item_local_pos_scaled.x, 0, item_local_pos_scaled.y) * item_detector_range
	return item_world_pos

func clear_loaded_items():
	pass

func place_item_on_ground(item: ItemUI):
	var new_item = ItemDB.create_item_for_world(item.id)
	get_tree().get_root().add_child(new_item)
	new_item.global_transform.origin = item_bag_to_world_pos(item)
	item.item_world_ref = new_item

func pick_up_item_from_ground(item: ItemUI):
	item.item_world_ref.queue_free()
