# controls all inventory containers, hotkeys, item manipulation, stats display

extends CanvasLayer

onready var grid_bkpk = $GridBackPack
onready var ground = $Ground
onready var item_info_pop_up = $InfoPopUp
onready var main_hand = $Hotkeys/MainHand #will be child of hotkeys
onready var hotkeys = $Hotkeys.get_children()
onready var stats_info = $Stats
onready var external_grid_container = $ExternalGridContainer
onready var external_bag_container = $ExternalBagContainer
onready var container_detector = get_parent().get_node("ContainerDetector")
onready var combat_manager = get_parent().get_node("CombatManager")

var is_open = false
var item_held = null
var item_offset = Vector2()
var last_container = null
var last_pos = Vector2()
var last_rot = false
var base_stats = {
	"hp_regen": 0,
	"hp_max" :10,
	"digestion": 0,
	"speed": 8
}
var cur_stats = {}
signal stats_updated

var external_items = []
var opened_world_container = null

var open_inv_sound = preload("./audio/open_inv.wav")
var close_inv_sound = preload("./audio/close_inv.wav")
var insert_item_squishy_sound = preload("./audio/goosh.wav")
var remove_item_squishy_sound = preload("./audio/grab_item_squishy.wav")

signal opened
signal closed

func _ready():
	#update_stats()
	for hotkey in hotkeys:
		hotkey.connect("hotkey_set", self, "update_combat_manager")

func update_combat_manager(hotkey_id, item_id):
	# TODO maybe replace this with a signal?
	combat_manager.equip_weapon(hotkey_id, item_id)

func grab_item(cursor_pos: Vector2):
	var c = get_container_under_cursor(cursor_pos)
	if c != null and c.has_method("grab_item"):
		item_held = c.grab_item(cursor_pos)
		if item_held != null:
			last_container = c
			last_pos = item_held.rect_global_position
			last_rot = item_held.rotated
			item_offset = item_held.rect_global_position - cursor_pos
			if item_held.get_parent() != self:
				item_held.get_parent().remove_child(item_held)
				add_child(item_held)
			move_child(item_held, get_child_count()) # move so it's visible on top of everything else
			update_stats()
			item_held.rect_global_position = cursor_pos + item_offset
			item_held.activate_physics(-item_offset)
			if is_squishy_container(last_container):
				play_sound(remove_item_squishy_sound)
			CursorManager.set_cursor("hand_closed")
	
func drag_item(cursor_pos: Vector2):
	if item_held != null:
		item_held.rect_global_position = cursor_pos + item_offset
	
func release_item():
	if item_held == null:
		return
	item_held.deactivate_physics()
	var insert_pos = item_held.rect_global_position + item_held.rect_size / 2
	var c = get_container_under_cursor(insert_pos)
	if c != null and c.has_method("insert_item") and c.insert_item(item_held):
		if c != grid_bkpk:
			item_held.clear_hotkey()
		item_held = null
		if is_squishy_container(c):
				play_sound(insert_item_squishy_sound)
	else:
		return_item()
		if is_squishy_container(last_container):
				play_sound(insert_item_squishy_sound)
	update_stats()
	CursorManager.set_cursor("hand_open")

func rotate_item(cursor_pos: Vector2):
	if item_held == null:
		return
	item_held.rotate(cursor_pos)
	item_offset = item_held.rect_global_position - cursor_pos

func display_item_info(cursor_pos: Vector2):
	#print(cursor_pos)
	var c = get_container_under_cursor(cursor_pos)
	if c != null and c.has_method("peek_item") and c.peek_item(cursor_pos) != null:
		item_info_pop_up.rect_size = Vector2()
		item_info_pop_up.text = c.peek_item(cursor_pos).description
		item_info_pop_up.show()
		item_info_pop_up.rect_global_position = cursor_pos - Vector2(0, item_info_pop_up.rect_size.y)
		move_child(item_info_pop_up, get_child_count())
	else:
		item_info_pop_up.hide()

func get_container_under_cursor(cursor_pos: Vector2) -> InventoryContainer:
	var containers = [grid_bkpk, ground]
	if last_container == grid_bkpk:
		containers += hotkeys
	else:
		containers += [main_hand]
	if external_bag_container.is_visible():
		containers += [external_bag_container]
	if external_grid_container.is_visible():
		containers += [external_grid_container]
	for c in containers:
		if c.get_global_rect().has_point(cursor_pos):
			return c
	return null

func return_item():
	if item_held == null:
		return
	item_held.deactivate_physics()
	if item_held.rotated != last_rot:
		item_held.rotate(Vector2())
	item_held.rect_global_position = last_pos
	last_container.insert_item(item_held)
	item_held = null

func update_stats():
	var equipped_items = grid_bkpk.get_all_items()
	for stat in base_stats:
		cur_stats[stat] = base_stats[stat]
	for item in equipped_items:
		if ItemDB.is_organ(item.id):
			var organ_stats = ItemDB.get_organ(item.id).stats
			for stat in organ_stats:
				if stat in cur_stats:
					cur_stats[stat] += organ_stats[stat]
				else:
					cur_stats[stat] = organ_stats[stat]
	var desc_text = "hp_max: " + str(cur_stats["hp_max"])
	desc_text += "\nhp_regen: " + str(cur_stats["hp_regen"])
	desc_text += "\nspeed: " + str(cur_stats["speed"])
	desc_text += "\ndigestion: " + str(cur_stats["digestion"])
	stats_info.text = desc_text
	emit_signal("stats_updated", cur_stats)

func open():
	if is_open:
		return
	CursorManager.set_cursor("hand_open")
	play_sound(open_inv_sound)
	is_open = true
	show_all()
	ground.load_nearby_items_on_ground()
	clear_external_items()
	external_bag_container.hide()
	external_grid_container.hide()
	var nearest_container = container_detector.get_nearest_container()
	opened_world_container = nearest_container
	if nearest_container != null:
		if nearest_container.type == "grid":
			external_grid_container.show()
			external_items = external_grid_container._load(nearest_container.contents, self)
			external_grid_container.get_node("Label").text = nearest_container.loot_table_id
		elif nearest_container.type == "bag":
			external_bag_container.show()
			external_items = external_bag_container._load(nearest_container.contents, self)
			external_bag_container.get_node("Label").text = nearest_container.loot_table_id
		else:
			print("WARNING unrecognized container type: ", nearest_container.type)
	emit_signal("opened")

func clear_external_items():
	for item in external_bag_container.items:
		item.queue_free()
	external_bag_container.items = []
	for item in external_grid_container.items:
		item.queue_free()
	external_grid_container.items = []

func close():
	if not is_open:
		return
	play_sound(close_inv_sound)
	is_open = false
	return_item()
	if opened_world_container != null:
		if external_bag_container.is_visible():
			opened_world_container.contents = external_bag_container._save()
		if external_grid_container.is_visible():
			opened_world_container.contents = external_grid_container._save()
	emit_signal("closed")
	hide_all()

func show_all():
	for child in get_children():
		if child.has_method("show"):
			child.show()
	item_info_pop_up.hide()

func hide_all():
	for child in get_children():
		if child.has_method("hide"):
			child.hide()

func _save():
	return_item()
	var bag_states = {}
	bag_states["grid_bkpk"] = grid_bkpk._save()
	bag_states["main_hand"] = ""
	if main_hand.item_in_slot != null:
		bag_states["main_hand"] = main_hand.item_in_slot.id
	return bag_states

func _load(bag_states):
	grid_bkpk._load(bag_states["grid_bkpk"], self)
	main_hand._load(bag_states["main_hand"], self)
	hide_all()
	update_stats()

func is_squishy_container(c: InventoryContainer) -> bool:
	return c in [grid_bkpk, external_grid_container]

func play_sound(sound):
	$AudioStreamPlayer.stream = sound
	$AudioStreamPlayer.play()