extends Control

var opened = false

const CLOSE_AFTER_TIME = 5.0
const FRAME_RATE = 0.07

var time_since_last_frame = 0.0
var time_since_opened = 0.0

onready var frames = $Frames.get_children()
onready var bit_mask_frames = $BitMaskFrames.get_children()
var cur_frame_ind = 0

enum ANIMATIONS {OPEN, CLOSE}
var cur_anim = ANIMATIONS.OPEN

func open():
	cur_anim = ANIMATIONS.OPEN
	set_frame(0)
	opened = true

func close():
	time_since_last_frame = 0.0
	time_since_opened = 0.0
	cur_frame_ind = 0
	opened = false
	hide_all_frames()

func hover(_item):
	cur_anim = ANIMATIONS.OPEN
	time_since_opened = 0.0

func _process(delta):
	if !opened:
		return
	time_since_last_frame += delta
	if time_since_last_frame > FRAME_RATE:
		process_frame()
		time_since_last_frame = 0.0
	
	time_since_opened += delta
	if time_since_opened > CLOSE_AFTER_TIME:
		cur_anim = ANIMATIONS.CLOSE

func process_frame():
	var dir = 1
	if cur_anim == ANIMATIONS.CLOSE:
		dir = -1
	cur_frame_ind = clamp(cur_frame_ind + dir, 0, frames.size() - 1)
	set_frame(cur_frame_ind)

func set_frame(ind):
	hide_all_frames()
	frames[ind].show()
	#$Light2D.texture = frames[ind].texture
	$Mask.texture = bit_mask_frames[ind].texture

func hide_all_frames():
	for frame in frames:
		frame.hide()
