extends Control

class_name Softbody2D

var texture = null
var size = Vector2()
var drag_point = Vector2(0.5, 0.5)
const NUM_OF_SEGMENTS = 8
var GRAVITY = 50
var SPRING_FORCE = 200
var DRAG = 0.1
var MAX_DIS = 24

var BASE_POINTS = [
		Vector2(0, 0),
		Vector2(0.5, 0),
		Vector2(1.0, 0),
		Vector2(1.0, 0.5),
		Vector2(1.0, 1.0),
		Vector2(0.5, 1.0),
		Vector2(0, 1.0),
		Vector2(0, 0.5),
	]
var edge_points_info = [] # array of dicts with base_point, uv, velo, cur_pos
var drag_offset = Vector2()

func _ready():
	set_physics_process(false)

func init(i_texture):
	texture = i_texture
	size = texture.get_size()
	rect_min_size = size
	rect_size = size

func start_moving(local_cursor_pos: Vector2):
	edge_points_info = []
	for i in NUM_OF_SEGMENTS:
		var edge_point_info = {}
		edge_point_info.base_pos = BASE_POINTS[i] * rect_size
		edge_point_info.uv = BASE_POINTS[i]
		edge_point_info.velo = Vector2()
		edge_point_info.cur_pos = to_global(edge_point_info.base_pos)
		edge_points_info.append(edge_point_info)
	drag_offset = local_cursor_pos
	drag_point = drag_offset / rect_size
	set_physics_process(true)

func stop_moving():
	set_physics_process(false)

func _physics_process(delta):
	run_physics(delta)

func run_physics(delta):
	for i in NUM_OF_SEGMENTS:
		var edge_point_info = edge_points_info[i]
		var cur_pos = edge_point_info.cur_pos
		var base_pos = to_global(edge_point_info.base_pos)
		var velo = edge_point_info.velo
		cur_pos += velo
		
		var dis_to_base = cur_pos.distance_to(base_pos)
		if dis_to_base != 0.0:
			var dir_to_base =  (base_pos - cur_pos) / dis_to_base
			if dis_to_base > MAX_DIS:
				cur_pos += dir_to_base * (dis_to_base - MAX_DIS)
				dis_to_base = MAX_DIS
			var cur_spring_force = dir_to_base * (SPRING_FORCE * dis_to_base / MAX_DIS) # spring force is stronger with distance
			velo += (Vector2.DOWN * GRAVITY + cur_spring_force) * delta - velo * DRAG
		edge_point_info.cur_pos = cur_pos
		edge_point_info.velo = velo
	update()

func _draw():
	for i in range(NUM_OF_SEGMENTS):
		var i1 = (i + 1) % NUM_OF_SEGMENTS
		var verts = [to_local(edge_points_info[i].cur_pos), to_local(edge_points_info[i1].cur_pos), drag_point * rect_size]
		var uv = [edge_points_info[i].uv, edge_points_info[i1].uv, drag_point]
		draw_polygon(PoolVector2Array(verts), PoolColorArray([]), PoolVector2Array(uv), texture)
		#draw_line(drag_point * rect_size, to_local(edge_points_info[i].cur_pos), Color.red)

func to_local(p_global: Vector2) -> Vector2:
	return get_global_transform().affine_inverse().xform(p_global)

func to_global(p_local: Vector2) -> Vector2:
	return get_global_transform().xform(p_local)