shader_type spatial;
render_mode unshaded;

uniform float x_stretch = 0.9;
uniform float y_stretch = 1.0;
uniform vec4 sky_color : hint_color;

uniform sampler2D cloud1_text : hint_albedo;
uniform vec4 cloud1_color : hint_color;
uniform float cloud1_scroll_spd = 0.0008;
uniform sampler2D cloud2_text : hint_albedo;
uniform vec4 cloud2_color : hint_color;
uniform float cloud2_scroll_spd = 0.0005;
uniform sampler2D cloud3_text : hint_albedo;
uniform vec4 cloud3_color : hint_color;
uniform float cloud3_scroll_spd = 0.0002;
uniform sampler2D sun_text : hint_albedo;
uniform vec4 sun_color : hint_color;
uniform float sun_scroll_spd = 0.0001;
uniform float sun_y_offset = 0.2;
uniform sampler2D displ_text : hint_albedo;
uniform float displ_amout = 0.1;
uniform float displ_speed = 0.00001;

void fragment(){
	float time = TIME * displ_speed;
	vec2 displ_amount = displ_amout * texture(displ_text, vec2(UV.x + time, UV.y + time * 0.5)).xy;
	float cloud1_a = round(texture(cloud1_text, vec2(x_stretch * UV.x + TIME * cloud1_scroll_spd, y_stretch * UV.y) + displ_amount).x);
	vec4 cloud1 = cloud1_a * cloud1_color;
	float clear_mask = 1.0 - cloud1_a;
	
	displ_amount = displ_amout * texture(displ_text, vec2(UV.x + time, UV.y + time * 0.5) + vec2(0.2)).xy;
	float cloud2_a = round(texture(cloud2_text, vec2(x_stretch * UV.x + TIME * cloud2_scroll_spd, y_stretch * UV.y) + displ_amount).x * clear_mask);
	vec4 cloud2 = cloud2_a * cloud2_color;
	clear_mask *= 1.0 - cloud2_a;
	
	displ_amount = displ_amout * texture(displ_text, vec2(UV.x + time, UV.y + time * 0.5) + vec2(0.5)).xy;
	float cloud3_a = round(texture(cloud3_text, vec2(x_stretch * UV.x + TIME * cloud3_scroll_spd, y_stretch * UV.y) + displ_amount).x * clear_mask);
	vec4 cloud3 = cloud3_a * cloud3_color;
	clear_mask *= 1.0 - cloud3_a;
	
	float sun_a = round(texture(sun_text, vec2(UV.x + TIME * sun_scroll_spd, sun_y_offset + y_stretch * UV.y) + displ_amount * 0.001).x * clear_mask);
	vec4 sun = sun_a * sun_color;
	clear_mask *= 1.0 - sun_a;
	
	vec4 sky = sky_color * clear_mask; 
	ALBEDO = (cloud1 + cloud2 + cloud3 + sun + sky).xyz;
}
