extends MeshInstance

export var stretchiness = 1.1
export var seg_len = .5
var max_stretch_len = 1
var width = 0
var height = 0
var length_of_longest_side = 0
onready var point1 = $Position3D.transform.origin
onready var point2 = $Position3D2.transform.origin
onready var point3 = $Position3D3.transform.origin
onready var point4 = $Position3D4.transform.origin

export var color = Color(1, 1, 1)

var points = []

func _ready():
	max_stretch_len = seg_len * stretchiness
	var p1p2len = (point2 - point1).length()
	var p3p4len = (point4 - point3).length()
	if p1p2len > p3p4len:
		height = int(p1p2len / seg_len)
		length_of_longest_side = p1p2len
	else:
		height = int(p3p4len / seg_len)
		length_of_longest_side = p3p4len
	
	var p1p3len = (point3 - point1).length()
	var p2p4len = (point4 - point2).length()
	if p1p3len > p2p4len:
		width = int(p1p3len / seg_len)
	else:
		width = int(p2p4len / seg_len)
	
	if p1p3len > length_of_longest_side:
		length_of_longest_side = p1p3len
	if p2p4len > length_of_longest_side:
		length_of_longest_side = p2p4len
	
	var p1p2_edge_points = get_line_of_points(point1, point2, height)
	var p3p4_edge_points = get_line_of_points(point3, point4, height)
	for i in range(height + 1):
		points.append(get_line_of_points(p1p2_edge_points[i], p3p4_edge_points[i], width))  
	
	for x in range(height + 1):
		for y in range(width + 1):
			if is_corner_point(x, y):
				continue
			points[x][y] -= Vector3(0, 3, 0)
	
	for _i in range(12):
		simulate_grid()
	
	clamp_height()
	render_static()

func is_corner_point(x, y):
	if x == 0 and y == 0:
		return true
	if x == height and y == width:
		return true
	if x == 0 and y == width:
		return true
	if x == height and y == 0:
		return true
	return false

func is_edge_point(x, y):
	return x == 0 or y == 0 or x == height or y == width

func get_line_of_points(start_point, end_point, num_of_segments):
	var edge_points = []
	var dir_to_end = end_point - start_point
	for i in range(num_of_segments + 1):
		edge_points.append(start_point + (dir_to_end / num_of_segments) * i)
	return edge_points

func simulate_grid():
	for i in range(width + 1):
		simulate_line(height + 1, true, i, true)
		simulate_line(height + 1, false, i, true)
	for i in range(height + 1):
		simulate_line(width + 1, true, i, false)
		simulate_line(width + 1, false, i, false)

 # start_index and end_index are both either on x or y, as determined by moving_along_x
# line_index refers to location on the other axis
func simulate_line(num_of_points, forwards, line_index, moving_along_x=false):
	#var len_from_start_to_end = 0
	var custom_stretch_amnt = max_stretch_len
	
#	if moving_along_x:
#		len_from_start_to_end = points[0][line_index].distance_to(points[num_of_points - 1][line_index])
#	else:
#		len_from_start_to_end = points[line_index][0].distance_to(points[line_index][num_of_points - 1])
#	custom_stretch_amnt *= len_from_start_to_end / length_of_longest_side
	var last_vec = null
	var range_iter = range(0, num_of_points)
	if not forwards:
		range_iter = range(num_of_points - 1, -1, -1)
	var point_index_x = 0
	var point_index_y = 0
	for i in range_iter:
		var cur_vec = null
		if moving_along_x:
			point_index_x = i
			point_index_y = line_index
		else:
			point_index_x = line_index
			point_index_y = i
		cur_vec = points[point_index_x][point_index_y]
		if i != 0 and i != num_of_points - 1 and last_vec != null:
			var dir_to_last = last_vec - cur_vec
			var dis_to_last = dir_to_last.length()
			if dis_to_last > custom_stretch_amnt:
				cur_vec += (dis_to_last - custom_stretch_amnt) * dir_to_last / dis_to_last
			points[point_index_x][point_index_y] = cur_vec
		last_vec = cur_vec

func clamp_height():
	for x in range(height + 1):
		for y in range(width + 1):
			if points[x][y].y < 0:
				points[x][y].y = 0

func render_static():
	var tmpMesh = Mesh.new()
	#var vertices = PoolVector3Array()
	#var UVs = PoolVector2Array()
	var mat = SpatialMaterial.new()
	
	mat.albedo_color = color
	mat.flags_unshaded = true
	
	var st = SurfaceTool.new()
	st.begin(Mesh.PRIMITIVE_TRIANGLES)
	st.set_material(mat)
	
	for x in range(height):
		for y in range(width):
			st.add_vertex(points[x][y])
			st.add_vertex(points[x + 1][y])
			st.add_vertex(points[x][y + 1])
			st.add_vertex(points[x + 1][y])
			st.add_vertex(points[x + 1][y + 1])
			st.add_vertex(points[x][y + 1])
	
	st.commit(tmpMesh)
	
	mesh = tmpMesh