# this has the stats info and paths to resources ( e.g. models, textures)
# for every item in the game

extends Node

const ITEM_FOLDER_PATH = "res://items/"

const WEAPON_RESOURCE_PATH = ITEM_FOLDER_PATH + "weapons/"
const WEAPON_TEXTURE_PATH = WEAPON_RESOURCE_PATH + "textures/"

const ORGAN_RESOURCE_PATH = ITEM_FOLDER_PATH + "organs/"
const ORGAN_TEXTURE_PATH = ORGAN_RESOURCE_PATH + "textures/"

const item_ui = preload("res://items/ItemUI.tscn")
const item_world = preload("res://items/ItemWorld.tscn")


const WEAPONS = { 
	"knife": {
		"icon": WEAPON_TEXTURE_PATH + "1x2.png",
		"model": WEAPON_RESOURCE_PATH + "Knife.tscn",
		"hotkey_icon": WEAPON_TEXTURE_PATH + "1x1.png",
		"crosshair": "crosshair_slash",
		"damage": 5,
		"attack_rate": 0.4,
		"automatic": false,
		"x_size": 2,
		"y_size": 1,
		"description": "A knife"
	},
	"pistol": {
		"icon": WEAPON_TEXTURE_PATH + "1x2.png",
		"model": WEAPON_RESOURCE_PATH + "Pistol.tscn",
		"hotkey_icon": WEAPON_TEXTURE_PATH + "1x1.png",
		"crosshair": "crosshair_basic_cross",
		"damage": 6,
		"attack_rate": 0.1,
		"automatic": false,
		"clip_size": 8,
		"reload_time": 0.7,
		"x_size": 2,
		"y_size": 1,
		"description": "Basic handgun"
	},
	"smg": {
		"icon": WEAPON_TEXTURE_PATH + "smg.png",
		"model": WEAPON_RESOURCE_PATH + "Smg.tscn",
		"hotkey_icon": WEAPON_TEXTURE_PATH + "smg_hotkey_icon.png",
		"crosshair": "crosshair_basic_cross",
		"damage": 5,
		"attack_rate": 0.1,
		"automatic": true,
		"clip_size": 30,
		"reload_time": 0.9,
		"x_size": 2,
		"y_size": 5,
		"description": "Basic smg"
	},
	"error": {
		"icon": WEAPON_TEXTURE_PATH + "1x1.png",
		"model": WEAPON_RESOURCE_PATH + "Knife.tscn",
		"crosshair": "crosshair_basic_cross",
		"damage": 0,
		"attack_rate": 0.2,
		"automatic": false,
		"clip_size": -1,
		"x_size": 1,
		"y_size": 1,
		"description": "Error, You shouldn't be able to see this"
	}
}

# organs can affect multiple stats, just do {"hp_regen": 0.5, "speed": 0.5}
# TODO make negative stats work properly
const ORGANS = { 
	"heart": {
		"icon": ORGAN_TEXTURE_PATH + "heart.png",
		"model": ORGAN_RESOURCE_PATH + "Heart.tscn",
		"stats": {"hp_regen": 0.5},
		"x_size": 1,
		"y_size": 1,
		"description": "A healthy heart"
	},
	"lung_left": {
		"icon": ORGAN_TEXTURE_PATH + "lung_l.png",
		"model": ORGAN_RESOURCE_PATH + "Lung.tscn",
		"stats": {"speed": 3.0},
		"x_size": 1,
		"y_size": 3,
		"description": "A healthy lung"
	},
	"lung_right": {
		"icon": ORGAN_TEXTURE_PATH + "lung_r.png",
		"model": ORGAN_RESOURCE_PATH + "Lung.tscn",
		"stats": {"speed": 3.0},
		"x_size": 1,
		"y_size": 3,
		"description": "A healthy lung"
	},
	"intestines": {
		"icon": ORGAN_TEXTURE_PATH + "intestines.png",
		"model": ORGAN_RESOURCE_PATH + "Intestines.tscn",
		"stats": {"digestion": 5.0},
		"x_size": 3,
		"y_size": 2,
		"description": "A healthy digestive system"
	},
	"fat": {
		"icon": ORGAN_TEXTURE_PATH + "fat.png",
		"model": ORGAN_RESOURCE_PATH + "Fat.tscn",
		"stats": {"hp_max": 15},
		"x_size": 1,
		"y_size": 1,
		"description": "A blob of fat"
	},
	"error": {
		"icon": ORGAN_TEXTURE_PATH + "1x1.png",
		"model": ORGAN_RESOURCE_PATH + "Heart.tscn",
		"stats": {"hp_regen": 0.0, "hp_max": 1},
		"x_size": 1,
		"y_size": 1,
		"description": "Error, You shouldn't be able to see this"
	}
}

func get_item(item_id):
	if item_id in WEAPONS:
		return get_weapon(item_id)
	elif item_id in ORGANS:
		return get_organ(item_id)
	else:
		print ("item not found ", item_id)
		return get_organ(item_id)

func get_weapon(item_id):
	var item = null
	if item_id in WEAPONS:
		item = WEAPONS[item_id]
	else:
		print ("weapon not found ", item_id)
		item = WEAPONS["error"]
	item["type"] = "weapon"
	return item

func get_organ(item_id):
	var item = null
	if item_id in ORGANS:
		item = ORGANS[item_id]
	else:
		print ("organ not found ", item_id)
		item = ORGANS["error"]
	item["type"] = "organ"
	return item

func create_item_for_ui(item_id, parent): #have to set parent so rect_size doesn't break
	var new_item = item_ui.instance()
	parent.add_child(new_item)
	new_item.init(item_id)
	return new_item

func create_item_for_world(item_id):
	var world_item = item_world.instance()
	var new_item = load(get_item(item_id)["model"]).instance()
	world_item.add_child(new_item)
	world_item.id = item_id
	world_item.override_id = true
	return world_item

func create_weapon(item_id):
	if not is_weapon(item_id):
		print(item_id, " is not a weapon")
		return null
	var new_wep = load(get_weapon(item_id)["model"]).instance()
	return new_wep

func is_weapon(item_id):
	return item_id in WEAPONS

func is_organ(item_id):
	return item_id in ORGANS