# this is an arbitrary item that appears in the player's inventory

extends Control
class_name ItemUI

var id = ""
var x_size = 1
var y_size = 1
var rotated = false
var description = ""
var type = ""
var item_world_ref = null
var hotkey_slot = null
var hotkey_slot_id = -1
var excluded_fields_for_description = ["model", "icon", "hotkey_icon"]

onready var hard_texture = $TextureBase/TextureRect
onready var soft_texture = $TextureBase/Softbody2D
onready var texture_base = $TextureBase

onready var blood_drip = $BloodDrip
onready var blood_sprays = $BloodSprays.get_children()

func init(item_id):
	id = item_id
	var item = ItemDB.get_item(item_id)
	var new_desc = ""
	for field in item:
		if field in self:
			self.set(field, item[field])
		elif field == "stats":
			for stat in item[field]:
				new_desc += stat + ": " + str(item[field][stat]) + "\n"
		elif not field in excluded_fields_for_description:
			new_desc += field + ": " + str(item[field]) + "\n"
	description = item_id + "\n" + type + "\n" + new_desc + description
	var item_texture = load(item.icon)
	hard_texture.texture = item_texture
	rect_size = item_texture.get_size() * hard_texture.rect_scale
	deactivate_physics()
	soft_texture.init(item_texture)
	
	anchor_left = 0.5
	anchor_right = 0.5
	anchor_bottom = 0.5
	anchor_top = 0.5

func rotate(pos):
	if x_size == 1 and y_size == 1:
		return
	var offset = pos - rect_global_position
	if rotated:
		texture_base.set_rotation(0.0)
	else:
		texture_base.set_rotation(PI / 2.0)
	rotated = !rotated
	rect_min_size = Vector2(rect_size.y, rect_size.x)
	rect_size = rect_min_size
	if rotated:
		texture_base.rect_global_position = rect_global_position + Vector2(rect_min_size.x, 0)
	else:
		texture_base.rect_global_position = rect_global_position
	rect_global_position = Vector2(pos.x - offset.y, pos.y - offset.x)
	var tmp = x_size
	x_size = y_size
	y_size = tmp
	
	if type == "organ":
		update_particles_positions()
		spray_blood()

func clear_hotkey():
	if hotkey_slot == null:
		return
	var tmp_hotkey_slot = hotkey_slot
	hotkey_slot = null
	hotkey_slot_id = -1
	tmp_hotkey_slot.clear()

func get_x_size():
	return x_size

func get_y_size():
	return y_size

func get_item_world_ref():
	return item_world_ref

func get_hotkey_slot():
	return hotkey_slot

func get_hotkey_slot_id():
	return hotkey_slot_id

func update_particles_positions():
	blood_drip.position = rect_size / 2.0
	$BloodSprays.rect_position = rect_size / 2.0
	blood_sprays[0].process_material.emission_box_extents = Vector3(rect_size.x / 2.0, 5.0, 0.0)
	blood_drip.process_material.emission_box_extents = Vector3(rect_size.x / 2.0, 5.0, 0.0)

#turns on softbody physics and particle physics for organs
func activate_physics(local_cursor_pos: Vector2):
	if type != "organ":
		return
	soft_texture.show()
	hard_texture.hide()
	var drag_offset = local_cursor_pos
	if rotated:
		var tmp = drag_offset
		drag_offset.y = soft_texture.size.y - tmp.x
		drag_offset.x = tmp.y
	soft_texture.start_moving(drag_offset)
	update_particles_positions()
	blood_drip.emitting = true

func deactivate_physics():
	soft_texture.hide()
	hard_texture.show()
	soft_texture.stop_moving()
	blood_drip.emitting = false

var spray_ind = 0
func spray_blood():
	for blood_spray in blood_sprays:
		blood_spray.emitting = false
	blood_sprays[spray_ind].emitting = true
	spray_ind = (spray_ind + 1) % blood_sprays.size()
