# arbitrary weapon controller script, can be used for melee weapons as well

extends Spatial

export var id = ""

var damage = 1
var attack_rate = 0.2
var time_since_last_attack = 0
var automatic = true

var clip_size = 8 # set negative for infinite clip
var cur_clip = clip_size
var reload_time = 2.0
var time_since_started_reloading = 0.0
var reloading = false

var objects_to_ignore = []  # to prevent hitting wielder with own bullets
var held_by = ""
var fire_point = null

var crosshair = null

signal reloading
signal attack
signal clip_empty

func _ready():
	init(id)

func init(weapon_id):
	id = weapon_id
	var item = ItemDB.get_item(weapon_id)
	for field in item:
		if field in self:
			self.set(field, item[field])
	cur_clip = clip_size #TODO prevent instant reload when switching weapons, probably do something in player combat manager
	time_since_last_attack = attack_rate

func _physics_process(delta):
	time_since_last_attack += delta
	if reloading:
		time_since_started_reloading += delta
		if time_since_started_reloading >= reload_time:
			complete_reload()
 
func reload():
	if clip_size <= 0:
		return
	#print("reloading")
	emit_signal("reloading")
	reloading = true

func stop_reloading():
	reloading = false
	time_since_started_reloading = 0
 
func complete_reload():
	reloading = false
	time_since_started_reloading = 0
	cur_clip = clip_size
	time_since_last_attack = attack_rate
 
func single_attack():
	if automatic:
		return
	attack()

func auto_attack():
	if not automatic:
		return
	attack()

#TODO might want to pass in target position or some way to make sure bullets aim correctly?
func attack():
	if reloading:
		return
	if time_since_last_attack < attack_rate:
		return
	if cur_clip == 0:
		reload()
		emit_signal("clip_empty") # TODO connect an audio player to this to play a click sound
		return
	
	if cur_clip > 0:
		cur_clip -= 1
	time_since_last_attack = 0
	var params = {
		"damage": damage, 
		"held_by": held_by, 
		"objects_to_ignore": objects_to_ignore
	}
	if fire_point != null:
		params.fire_point = fire_point
	emit_signal("attack", params)

func set_objects_to_ignore(to_ignore):
	objects_to_ignore = to_ignore

func get_id():
	return id