# This manages the procedurally animated muzzle flash
# There are three bones in the mesh, each can be randomly scaled between a set range
# every time you call flash

extends Spatial

var flash_time = 0.04
var cur_flash_time = 0
var flashing = false
onready var skeleton = $Graphics/Armature/Skeleton

export var flash_back_scale_range = Vector2(1.0, 1.0)
export var flash_mid_scale_range = Vector2(1.0, 1.0)
export var flash_front_scale_range = Vector2(1.0, 1.0)

func _ready():
	$Graphics.hide()

func _process(delta):
	if flashing:
		cur_flash_time += delta
		if cur_flash_time >= flash_time:
			$Graphics.hide()
			flashing = false
			cur_flash_time = 0

func flash(_params={}): # need _params since gets called by a signal that passes this
	$Graphics.show()
	flashing = true
	set_bone_scale(0, flash_back_scale_range, true, true, false)
	set_bone_scale(2, flash_mid_scale_range, true, true, false)
	set_bone_scale(1, flash_front_scale_range, true, true, false)
	rotation_degrees.z = rand_range(0, 360)

func set_bone_scale(bone_id: int, scale_range: Vector2, affects_x=true, affects_y=true, affects_z=true):
	var scale_vector = Vector3(1, 1, 1)
	if affects_x:
		scale_vector.x = rand_range(scale_range.x, scale_range.y)
	if affects_y:
		scale_vector.y = rand_range(scale_range.x, scale_range.y)
	if affects_z:
		scale_vector.z = rand_range(scale_range.x, scale_range.y)
		
	skeleton.set_bone_pose(bone_id, Transform.IDENTITY.scaled(scale_vector))
