extends Spatial

var blood_decal = preload("BloodDecal.tscn")
var hit_decal = preload("HitDecal.tscn")

var blood_mat = preload("BloodDecalMat.tres")
var blood_floor_mat = preload("GroundBloodDecalMat.tres")
var hit_mat = preload("HitDecalMat.tres")

var blood_multi_mesh = null
var blood_floor_multi_mesh = null
var hit_multi_mesh = null
const MAX_DECALS = 100

func _ready():
	blood_multi_mesh = instance_multi_mesh(blood_mat)
	blood_floor_multi_mesh = instance_multi_mesh(blood_floor_mat)
	hit_multi_mesh = instance_multi_mesh(hit_mat)

func instance_multi_mesh(mat):
	var new_multi_mesh_inst = MultiMeshInstance.new()
	new_multi_mesh_inst.material_override = mat
	get_tree().get_root().call_deferred("add_child", new_multi_mesh_inst)
	
	var new_multi_mesh = MultiMesh.new()
	new_multi_mesh.mesh = QuadMesh.new()
	new_multi_mesh.transform_format = MultiMesh.TRANSFORM_3D
	new_multi_mesh.instance_count = MAX_DECALS
	new_multi_mesh.visible_instance_count = 0
	new_multi_mesh_inst.multimesh = new_multi_mesh
	return new_multi_mesh

func place_blood_decal(pos: Vector3, dir: Vector3, decal_scale=1.0):
	create_decal(blood_decal, pos, dir, Vector3(decal_scale,decal_scale,decal_scale), blood_multi_mesh)

func place_blood_floor_decal(pos: Vector3, dir: Vector3, decal_scale=1.0, rot=-1, custom_offset=Vector3()):
	create_decal(blood_decal, pos, dir, Vector3(2.5, 1.0, 1.0) * decal_scale, blood_floor_multi_mesh, rot, false, custom_offset)

func place_hit_decal(pos: Vector3, dir: Vector3, decal_scale=1.0):
	create_decal(hit_decal, pos, dir, Vector3(decal_scale,decal_scale,decal_scale), hit_multi_mesh)

func create_decal(obj, pos: Vector3, dir: Vector3, decal_scale: Vector3, multi_mesh: MultiMesh, rot=0.0, rand_rot=true, custom_offset=Vector3()):
	var bd = obj.instance()
	get_tree().get_root().add_child(bd)
	bd.global_transform.origin = pos + dir * 0.005 + custom_offset
	
	var x = dir.cross(Vector3.UP).normalized()
	if x == Vector3.ZERO:
		x = dir.cross(Vector3.RIGHT).normalized()
	var y = dir.cross(x).normalized()
	bd.global_transform.basis = Basis(x,y, dir)
	
	bd.scale = decal_scale
	if rand_rot:
		bd.rotate(dir, rand_range(0.0, PI))
	else:
		bd.rotate(dir, rot)

	multi_mesh.visible_instance_count += 1
	multi_mesh.set_instance_transform((multi_mesh.visible_instance_count-1) % MAX_DECALS, bd.global_transform)
	bd.queue_free()